@@ -6,6 +6,7 @@ from urllib.parse import quote as urlquote
 # import requests  # we use lazy imports to improve load time for local commands
 # from bs4 import BeautifulSoup
 # from bs4.element import NavigableString
+from tqdm import tqdm
 
 from kks.util.h2t import HTML2Text
 
@@ -15,6 +16,9 @@ CONTEST_ID_BY_GROUP = {
 }
 
 
+PROBLEM_INFO_VERSION = 2
+
+
 class Links:
     WEB_CLIENT_ROOT = 'https://caos.ejudge.ru/ej/client'
     SETTINGS = f'{WEB_CLIENT_ROOT}/view-settings/S__SID__'
@@ -27,6 +31,7 @@ class Links:
 
 class Status:
     OK = 'OK'
+    OK_AUTO = 'OK (auto)'  # only in summary
     REVIEW = 'Pending review'
     CHECK = 'Pending check'
     TESTING = 'Accepted for testing'
@@ -44,18 +49,26 @@ class AuthData:
 
 
 class BaseProblem:
-    def __init__(self, short_name, name, href, status, tests_passed, score):
+    def __init__(self, short_name, href):
         self.short_name = short_name
-        self.name = name
         self.href = href
+
+    def contest(self):
+        return extract_contest_name(self.short_name)
+
+
+class SummaryProblem(BaseProblem):
+    def __init__(self, short_name, name, href, status, tests_passed, score):
+        super().__init__(short_name, href)
+        self.name = name
         self.status = status
         self.tests_passed = tests_passed
         self.score = score
 
 
-class Problem(BaseProblem):
+class Problem(SummaryProblem):
     def color(self):
-        return 'green' if self.status == Status.OK \
+        return 'green' if self.status in [Status.OK, Status.OK_AUTO] \
             else 'green' if self.status == Status.REVIEW \
             else 'white' if self.status == Status.NOT_SUBMITTED \
             else 'bright_yellow' if self.status == Status.CHECK \
@@ -63,7 +76,7 @@ class Problem(BaseProblem):
             else 'red'
 
     def bold(self):
-        return self.status == Status.OK
+        return self.status in [Status.OK, Status.OK_AUTO]
 
     def get_full(self, session):
         # TODO use API? (see #68)
@@ -71,6 +84,16 @@ class Problem(BaseProblem):
         return FullProblem(self, page)
 
 
+class CacheKeys:
+    @staticmethod
+    def penalty(contest):
+        return f'p_{contest}'
+
+    @staticmethod
+    def deadline(contest):
+        return f'dl_{contest}'
+
+
 class Submission:
     def __init__(self, row):
         cells = row.find_all('td')
@@ -188,9 +211,10 @@ class StandingsRow:
 
 
 class Standings:
-    def __init__(self, tasks, rows):
+    def __init__(self, tasks, rows, user=None):
         self.tasks = tasks
         self.rows = rows
+        self.user = user
 
         self.contests = [
             contest
@@ -202,17 +226,30 @@ class Standings:
             for contest, tasks in groupby(self.tasks, lambda task: task.contest)
         }
 
+    def fix_is_self(self, user, contest_id):
+        for row in self.rows:
+            row.is_self = row.user == user and row.contest_id == contest_id
+
+
+class Deadlines:
+    def __init__(self, soft, hard):
+        self.soft = soft
+        self.hard = hard
+
 
 class ProblemInfo:
     """Subset of task info table used for max score estimation"""
-
-    def __init__(self, full_score, run_penalty, current_penalty):
+    def __init__(self, full_score, run_penalty, current_penalty, deadlines):
         self.full_score = full_score
         self.run_penalty = run_penalty
         self.current_penalty = current_penalty
+        self.deadlines = deadlines
+
+    def past_deadline(self):
+        return self.deadlines.hard is not None and datetime.now() > self.deadlines.hard or self.current_penalty >= self.full_score
 
 
-class FullProblem(BaseProblem):
+class FullProblem(SummaryProblem):
     keep_info = ['Time limit:', 'Real time limit:', 'Memory limit:']
 
     def __init__(self, problem, page):
@@ -397,7 +434,6 @@ def ejudge_standings(session):
     from bs4 import BeautifulSoup
 
     page = session.get(Links.USER_STANDINGS)
-
     soup = BeautifulSoup(page.content, 'html.parser')
 
     title = soup.find(class_='main_phrase').text
@@ -416,22 +452,35 @@ def ejudge_standings(session):
 
     def parse_rows():
         for row in rows:
-            user = row.find(class_='st_team').text
-            score_cells = row.find_all(class_='st_prob')
+            cells = row.find_all('td')  # search in html only once
+            score_cells = [c for c in cells if 'st_prob' in c['class']]
+            user = None
+            place = None
+            total = None
+            score = None
+            for c in cells:
+                if 'st_team' in c['class']:
+                    user = c.text
+                if 'st_place' in c['class']:
+                    place = c.text
+                elif 'st_total' in c['class']:
+                    total = c.text
+                elif 'st_score' in c['class']:
+                    score = c.text
 
             yield StandingsRow(
-                row.find(class_='st_place').text,
+                place,
                 user,
                 [
                     to_task_score(task.contest, cell)
                     for task, cell in zip(tasks, score_cells)
                 ],
-                int(row.find(class_='st_total').text),
-                int(row.find(class_='st_score').text),
+                int(total),
+                int(score),
                 name == user
             )
 
-    return Standings(tasks, list(parse_rows()))
+    return Standings(tasks, list(parse_rows()), name)
 
 
 def extract_contest_name(task_name):
@@ -484,6 +533,33 @@ def ejudge_report(link, session):
     return Report(comments, tests)
 
 
+def update_cached_problems(cache, names, session, only_contests=False, summary=None):
+
+    def cached_problem(problem):
+        return BaseProblem(problem.short_name, problem.href)
+
+    problem_list = cache.get('problem_links', [])  # we can avoid loading summary
+    if len(problem_list) != len(names) or \
+            any(problem.short_name != name for problem, name in zip(problem_list, names)):
+        problem_list = summary or ejudge_summary(session)
+        problem_list = [cached_problem(p) for p in problem_list]
+        cache.set('problem_links', problem_list)
+
+    if only_contests:
+        first_problems = []
+        for contest, problems in groupby(problem_list, lambda problem: problem.contest()):
+            first_problems.append(next(problems))
+            problem_list = first_problems
+
+    with tqdm(total=len(problem_list), leave=False) as pbar:
+        def with_progress(func, *args, **kwargs):
+            result = func(*args, **kwargs)
+            pbar.update(1)
+            return result
+
+        return [with_progress(get_problem_info, problem, cache, session) for problem in problem_list]
+
+
 def get_problem_info(problem, cache, session):
     from bs4 import BeautifulSoup
 
@@ -497,42 +573,38 @@ def get_problem_info(problem, cache, session):
     full_score = full_scores.get(problem.short_name)
     run_penalty = run_penalties.get(problem.short_name)
 
-    penalty_key = f'p_{problem.contest}'
+    penalty_key = CacheKeys.penalty(problem.contest())
     current_penalty = cache.get(penalty_key)
 
-    if full_score is None or run_penalty is None or current_penalty is None:  # new problem or need to update penalty
-        need_loading = True
+    deadline_key = CacheKeys.deadline(problem.contest())
+    deadlines = cache.get(deadline_key)
+
+    need_loading = any(field is None for field in [full_score, run_penalty, current_penalty, deadlines])  # new problem or need to update penalty
 
     if not need_loading:
-        return ProblemInfo(full_score, run_penalty, current_penalty)
-
-    now = datetime.now()
-    def update_cache(full_score, run_penalty, current_penalty, soft_dl, hard_dl):
-        past_deadline = hard_dl is not None and now > hard_dl
-
-        if past_deadline:
-            # see issue #72
-            full_score = 0
-            current_penalty = 0
+        return ProblemInfo(full_score, run_penalty, current_penalty, deadlines)
 
+    def update_cache(full_score, run_penalty, current_penalty, deadlines):
+        result = ProblemInfo(full_score, run_penalty, current_penalty, deadlines)
         full_scores[problem.short_name] = full_score
         run_penalties[problem.short_name] = run_penalty
         cache.set('full', full_scores)
         cache.set('run', run_penalties)
 
-        if current_penalty >= full_score and current_penalty != 0 or past_deadline:
+        if result.past_deadline():
             expiration = None
             # max_score = 0, will not change
-            # NOTE may need patching for kr contests
         else:
-            if soft_dl is not None:
-                expiration = soft_dl
+            if deadlines.soft is not None:
+                expiration = deadlines.soft
             else:
                 expiration = timedelta(days=4, hours=23, minutes=55)
 
         cache.set(penalty_key, current_penalty, expiration)
+        # deadlines may be shifted or there may (?) be multiple soft deadlines, so they should expire too
+        cache.set(deadline_key, deadlines, expiration)
 
-        return ProblemInfo(full_score, run_penalty, current_penalty)
+        return result
 
     page = session.get(problem.href)
     soup = BeautifulSoup(page.content, 'html.parser')
@@ -542,12 +614,12 @@ def get_problem_info(problem, cache, session):
     full_score = 0
     run_penalty = 0  # may be incorrect for kr
     current_penalty = 0
-    soft_dl = None
-    hard_dl = None
+    deadlines = Deadlines(None, None)
 
     if problem_info is None:
-        # may happen in kr contests?
-        return update_cache(full_score, run_penalty, current_penalty, None)
+        # if this ever happens, we assume the problem is past the deadline
+        deadlines.hard = datetime.fromtimestamp(0)
+        return update_cache(full_score, run_penalty, current_penalty, deadlines)
 
     # TODO "Full score" can be missing (sm01-3)
     # in this case we should use max score from the table as full
@@ -560,11 +632,11 @@ def get_problem_info(problem, cache, session):
         elif key.text == 'Current penalty:':
             current_penalty = int(value.text)
         elif key.text == 'Next soft deadline:':
-            soft_dl = datetime.strptime(value.text, '%Y/%m/%d %H:%M:%S')
+            deadlines.soft = datetime.strptime(value.text, '%Y/%m/%d %H:%M:%S')
         elif key.text == 'Deadline:':
-            hard_dl = datetime.strptime(value.text, '%Y/%m/%d %H:%M:%S')
+            deadlines.hard = datetime.strptime(value.text, '%Y/%m/%d %H:%M:%S')
 
-    return update_cache(full_score, run_penalty, current_penalty, soft_dl, hard_dl)
+    return update_cache(full_score, run_penalty, current_penalty, deadlines)
 
 
 def chunks(iterable, chunk_size):
